//
// C++ Implementation: genericsync
//
// Description:
//
//
// Author: nlm <nlm@hsc.ge>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "../Includes/qstdinc.h"
#include "../Includes/qsync.h"
#include "../Includes/qlogger.h"

QLOG_REGISTER_SOURCE

QMutexSync::QMutexSync()
{
#ifndef _WIN32
	pthread_mutexattr_t 	m_attr;
	if (pthread_mutexattr_init(&m_attr) ||
		pthread_mutexattr_settype(&m_attr, PTHREAD_MUTEX_RECURSIVE) ||
		pthread_mutex_init(&m_mutex, &m_attr) ||
		pthread_mutexattr_destroy(&m_attr))
	{
		QLogError(0, "Error initializing mutex: %d", errno);
		raise(SIGTRAP);
	}
#else
	InitializeCriticalSection(&m_mutex);
#endif
}

QMutexSync::~QMutexSync()
{
#ifndef _WIN32
	if (pthread_mutex_destroy(&m_mutex))
	{
		QLogError(0, "Error destroying mutex: %d", errno);
		raise(SIGTRAP);
	}
#else
	DeleteCriticalSection(&m_mutex);
#endif
}

void QMutexSync::SyncLock()
{
#ifndef _WIN32
	if (pthread_mutex_lock(&m_mutex))
	{
		QLogError(0, "Error locking mutex: %d", errno);
		raise(SIGTRAP);
	}
#else
	EnterCriticalSection(&m_mutex);
#endif
}

void QMutexSync::SyncUnlock()
{
#ifndef _WIN32
	if (pthread_mutex_unlock(&m_mutex))
	{
		QLogError(0, "Error unlocking mutex: %d", errno);
		raise(SIGTRAP);
	}
#else
	LeaveCriticalSection(&m_mutex);
#endif
}

QBar::QBar()
{
	m_bBar = m_bAck = false;
}

QBar::~QBar()
{
}
	
void QBar::Bar()
{
	QLocker lock(this);
	m_bBar = true;
	m_bAck = false;
}

void QBar::ReleaseBar()
{
	QLocker lock(this);
	m_bBar = m_bAck = false;
}

bool QBar::WaitBarAck()
{
	bool bRetVal = false;
	
	for (int i=0; i<200; i++)
	{
		{
			QLocker lock(this);
			if (m_bAck)
			{
				bRetVal = true;
				break;
			}
		}
#ifndef _WIN32
		usleep(10000);
#else
		::Sleep(10);
#endif
	}
	
	return bRetVal;
}
	
bool QBar::IsBarred()
{
	QLocker lock(this);
	return m_bBar;
}

bool QBar::IsBarredNL()
{
	return m_bBar;
}

void QBar::AckBar()
{
	QLocker lock(this);
	m_bAck = true;
}
